from django.conf import settings

ZARINPAL_WEBSERVICE = "https://www.zarinpal.com/pg/services/WebGate/wsdl"
ZARINPAL_START_GATEWAY = "https://www.zarinpal.com/pg/StartPay/"
ZARINPAL_CALLBACK_URL = getattr(settings, "ZARINPAL_CALLBACK_URL", None)
ZARINPAL_SIMULATION = getattr(settings, "ZARINPAL_SIMULATION", False)
ZARINPAL_MERCHANT_ID = getattr(settings, "ZARINPAL_MERCHANT_ID", "")

if ZARINPAL_SIMULATION:
    ZARINPAL_WEBSERVICE = "https://sandbox.zarinpal.com/pg/services/WebGate/wsdl"
    ZARINPAL_START_GATEWAY = "https://sandbox.zarinpal.com/pg/StartPay/"
    ZARINPAL_MERCHANT_ID = "XXXXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX"

TRANSACTION_STATUS_CHOICES = [
    ("PENDING", "Transaction has just started"),
    ("INCOMPLETE_PARAMETERS", "Transaction parameters were incomplete"),
    ("MERCHANT_CODE_INCORRECT", "Merchant code was incorrect"),
    ("MERCHANT_CODE_NOT_ACTIVE", "Merchant code is not active"),
    ("AUTHORITY_CODE_INVALID", "Authority code was invalid"),
    ("ALREADY_VERIFIED", "Transaction was already verified"),
    ("FAILED", "Transaction was failed"),
    ("UNKNOWN", "Transaction was failed with unknown error"),
    ("SUCCESS", "Transaction was successfully done"),
]
TRANSACTION_STATUS_CODES = {
    -1: "INCOMPLETE_PARAMETERS",
    -2: "MERCHANT_CODE_INCORRECT",
    -3: "MERCHANT_CODE_NOT_ACTIVE",
    -8: "AUTHORITY_CODE_INVALID",
    -9: "ALREADY_VERIFIED",
    -10: "FAILED",
    -100: "UNKNOWN",
    100: "SUCCESS",
}



from django.db.models import manager

from .config import ZARINPAL_SIMULATION


class TransactionManager(manager.Manager):
    """ Manager for :class:`Transaction` """

    def create_transaction(self, transaction_data):
        transaction_data["status"] = "PENDING"
        transaction_data["simulation"] = ZARINPAL_SIMULATION
        created_transaction = self.create(**transaction_data)
        created_transaction.order_number = created_transaction.id
        created_transaction.save(update_fields=["order_number"])
        return created_transaction



from zeep import Client
import logging

from zarinpal.helpers import generate_start_transaction_data
from .config import (
    ZARINPAL_START_GATEWAY,
    ZARINPAL_WEBSERVICE,
)
from .models import Transaction


def start_transaction(transaction_data: dict) -> str:
    transaction = Transaction.objects.create_transaction(transaction_data)
    start_transaction_data = generate_start_transaction_data(transaction)
    client = Client(ZARINPAL_WEBSERVICE)
    result = client.service.PaymentRequest(
        start_transaction_data["merchant_id"],
        start_transaction_data["amount"],
        start_transaction_data["description"],
        start_transaction_data["email"],
        start_transaction_data["mobile"],
        start_transaction_data["callback_url"],
    )
    print(start_transaction_data)
    if result.Status == 100:
        transaction.authority = result.Authority
        transaction.save(update_fields=["authority"])
        return ZARINPAL_START_GATEWAY + result.Authority
    else:
        logging.error(result)



from zarinpal.config import ZARINPAL_WEBSERVICE, ZARINPAL_MERCHANT_ID, ZARINPAL_CALLBACK_URL
from zarinpal.exceptions import TransactionDoesNotExist
from zarinpal.models import Transaction
from zeep import Client


def verify_transaction(status: str, authority: int) -> Transaction:
    client = Client(ZARINPAL_WEBSERVICE)
    try:
        transaction = Transaction.objects.get(status="PENDING", authority=authority)
    except Transaction.DoesNotExist:
        raise TransactionDoesNotExist
    if status == "OK":
        result = client.service.PaymentVerification(
            ZARINPAL_MERCHANT_ID, authority, transaction.amount
        )
        if result.Status == 100:
            transaction.success(result.RefID)
        elif result.Status == 101:
            transaction.status = result.Status
            return HttpResponse("Transaction submitted : " + str(result.Status))
        else:
            transaction.fail(result.Status)
    else:
        transaction.fail("Canceled")

    return transaction


def generate_start_transaction_data(transaction):
    return {
        "merchant_id": ZARINPAL_MERCHANT_ID,
        "amount": transaction.amount,
        "description": transaction.description,
        "email": transaction.email,
        "mobile": transaction.mobile,
        "callback_url": get_callback_url(),
    }


def get_callback_url():
    if ZARINPAL_CALLBACK_URL:
        return ZARINPAL_CALLBACK_URL
    else:
        return (
            "http://"
            + Site.objects.get_current().domain
            + reverse("zarinpal:verify_transaction")
        )



# -*- coding: utf-8 -*-
from django.http import HttpResponse
from django.shortcuts import redirect
from zeep import Client

from zarinpal.config import ZARINPAL_WEBSERVICE, ZARINPAL_MERCHANT_ID
from .models import Transaction

from zarinpal.helpers import verify_transaction
from .signals import transaction_verified


def verify_transaction_view(request):
    authority = request.GET.get("Authority")
    status = request.GET.get("Status")
    transaction = verify_transaction(status, authority)
    transaction_verified.send(sender=None, transaction=transaction)
    print("verified")
    if transaction.callback_url:
        return redirect(
            transaction.callback_url
            + f"/?success={transaction.is_successful()}&orderNumber={transaction.order_number}"
        )
    return HttpResponse(f"done transaction{authority}")
    # return redirect(
    #    transaction.callback_url
    # )



# Generated by Django 2.1.7 on 2019-04-12 19:50

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import hashid_field.field


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Transaction',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('amount', models.DecimalField(blank=True, decimal_places=2, default=0, max_digits=64, null=True)),
                ('authority', models.CharField(blank=True, max_length=100, null=True)),
                ('ref_id', models.IntegerField(blank=True, null=True)),
                ('description', models.TextField()),
                ('callback_url', models.CharField(max_length=100)),
                ('first_name', models.CharField(blank=True, max_length=225, null=True)),
                ('last_name', models.CharField(blank=True, max_length=225, null=True)),
                ('email', models.CharField(blank=True, max_length=225, null=True)),
                ('mobile', models.CharField(blank=True, max_length=225, null=True)),
                ('order_number', hashid_field.field.HashidField(alphabet='abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890', blank=True, min_length=7, null=True)),
                ('address', models.CharField(blank=True, max_length=225, null=True)),
                ('country', models.CharField(blank=True, max_length=225, null=True)),
                ('postal_code', models.CharField(blank=True, max_length=225, null=True)),
                ('city', models.CharField(blank=True, max_length=100, null=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('successful_payment_date_time', models.DateTimeField(blank=True, null=True)),
                ('status', models.CharField(max_length=100)),
                ('failure_reason', models.CharField(blank=True, max_length=100, null=True)),
                ('simulation', models.BooleanField(default=False)),
                ('user', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



# -*- coding: utf-8 -*-
from django.urls import path

from .views import verify_transaction_view

app_name = "zarinpal"
urlpatterns = [path("verify/", verify_transaction_view, name="verify_transaction")]



class Error(Exception):
    """
    BaseClass for exceptions
    """


class UnknownTransactionFailure(Error):
    """ Unknown transaction failure """


class UnknownTransactionStatusCode(Error):
    """ Unknown transaction status code """


class CallbackUrlNotProvided(Error):
    """ callback url is not provided in transaction data or project settings """


class TransactionDoesNotExist(Error):
    """ No transaction submitted with this authority """



# -*- coding: utf-8 -*-
from django.contrib.auth import get_user_model
from django.db import models
from django.urls import reverse
from django.utils import timezone
from hashid_field import HashidField
from django.conf import settings

from .exceptions import CallbackUrlNotProvided
from .config import TRANSACTION_STATUS_CHOICES, ZARINPAL_START_GATEWAY
from .managers import TransactionManager


class Transaction(models.Model):
    user = models.ForeignKey(get_user_model(), on_delete=models.CASCADE, null=True)
    amount = models.DecimalField(
        max_digits=64, decimal_places=2, default=0, blank=True, null=True
    )
    authority = models.CharField(max_length=100, blank=True, null=True)
    ref_id = models.IntegerField(null=True, blank=True)
    description = models.TextField()
    callback_url = models.CharField(max_length=100)
    first_name = models.CharField(max_length=225, blank=True, null=True)
    last_name = models.CharField(max_length=225, blank=True, null=True)
    email = models.CharField(max_length=225, blank=True, null=True)
    mobile = models.CharField(max_length=225, blank=True, null=True)
    order_number = HashidField(
        allow_int_lookup=True, blank=True, null=True, salt=getattr(settings, "secret_key", None)
    )
    address = models.CharField(max_length=225, blank=True, null=True)
    country = models.CharField(max_length=225, blank=True, null=True)
    postal_code = models.CharField(max_length=225, blank=True, null=True)
    city = models.CharField(max_length=100, blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True)
    successful_payment_date_time = models.DateTimeField(blank=True, null=True)
    # status = models.CharField(max_length=100, choices=TRANSACTION_STATUS_CHOICES)
    status = models.CharField(max_length=100)
    failure_reason = models.CharField(max_length=100, blank=True, null=True)
    simulation = models.BooleanField(default=False)
    objects = TransactionManager()

    def __repr__(self):
        return "<zarinpal id:{0}>".format(self.order_number)

    def __str__(self):
        return "zarinpal: {0}".format(self.order_number)

    def success(self, ref_id):
        self.ref_id = ref_id
        self.status = "SUCCESS"
        self.successful_payment_date_time = timezone.now()
        self.save(update_fields=["status", "successful_payment_date_time", "ref_id"])

    def fail(self, failure_reason=None):
        self.status = "FAILED"
        if failure_reason:
            self.failure_reason = failure_reason
        self.save(update_fields=["status", "failure_reason"])

    def is_successful(self):
        return self.status == "SUCCESS"

    def get_transaction_start_url(self, request=None):
        if self.simulation is False:
            return ZARINPAL_START_GATEWAY + self.authority
        else:
            relative_start_url = reverse(
                "zarinpal:sandbox-payment", kwargs={"authority_start": self.authority}
            )
            if request:
                return request.build_absolute_uri(relative_start_url)
            else:
                return relative_start_url

    def get_client_callback_url(self):
        if self.callback_url:
            return self.callback_url + f"?orderNumber={self.order_number}"
        else:
            raise CallbackUrlNotProvided(
                f"Callback url is not set in transaction with order number {self.order_number.hashid}"
            )
